from ocp_resources.constants import TIMEOUT_4MINUTES
from ocp_resources.resource import NamespacedResource


class Endpoints(NamespacedResource):
    """
    Endpoints object. API reference:
    https://docs.openshift.com/container-platform/4.12/rest_api/network_apis/endpoints-v1.html#endpoints-v1
    """

    api_version = NamespacedResource.ApiVersion.V1

    def __init__(
        self,
        name=None,
        namespace=None,
        client=None,
        addresses=None,
        ports=None,
        teardown=True,
        privileged_client=None,
        yaml_file=None,
        delete_timeout=TIMEOUT_4MINUTES,
        **kwargs,
    ):
        """
        Args:
            name (str): Name of the endpoints resource
            namespace (str): Namespace of endpoints resource
            client: (DynamicClient): DynamicClient for api calls
            addresses (list): List of ip addresses which offers the related ports that are marked as ready
            ports (list): List of port numbers available on the related ip addresses
            teardown (bool): Indicates if the resource should be torn down at the end
            privileged_client (DynamicClient): Privileged client for api calls
            yaml_file (str): yaml file for the resource.
            delete_timeout (int): timeout associated with delete action
        """
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            privileged_client=privileged_client,
            yaml_file=yaml_file,
            delete_timeout=delete_timeout,
            **kwargs,
        )
        self.addresses = addresses
        self.ports = ports

    def to_dict(self):
        super().to_dict()
        if not self.yaml_file:
            if not (self.addresses and self.ports):
                raise ValueError(
                    "yaml_file or parameters 'addresses' and 'ports' are required."
                )
            self.res.update(
                {
                    "subsets": {
                        "addresses": self.addresses,
                        "ports": self.ports,
                    }
                }
            )



# API reference: https://tekton.dev/docs/pipelines/pipelineruns/

from ocp_resources.resource import NamespacedResource


class PipelineRun(NamespacedResource):
    api_group = NamespacedResource.ApiGroup.TEKTON_DEV

    def __init__(
        self,
        name=None,
        namespace=None,
        pipelineref=None,
        params=None,
        service_account_name=None,
        client=None,
        yaml_file=None,
        **kwargs,
    ):
        """
        Args:
            name (str): Name of the pipelinerun
            namespace (str): Namespace of the pipelinerun
            pipelineref (str): Mandatory: Base pipeline to run pipelineruns
            client: (DynamicClient): DynamicClient to use.
            params (dict): Optional params to add during triggering a run.
            params can be set/changed based on pipelineref.
            example : params={"param_name1":"param_value1", "param_name2":"param_value2"}
            service_account_name (str): Optional to provide service account
        """
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            **kwargs,
        )
        self.pipelineref = pipelineref
        self.params = params
        self.service_account_name = service_account_name

    def to_dict(self):
        super().to_dict()
        if not self.yaml_file:
            if not self.pipelineref:
                raise ValueError("pipelineref is mandatory")
            self.res["spec"] = {}
            self.res["spec"]["pipelineref"] = {"name": self.pipelineref}

            if self.params:
                self.res["spec"]["params"] = [
                    {"name": params_name, "value": params_value}
                    for params_name, params_value in self.params.items()
                ]

            if self.service_account_name:
                self.res["spec"]["serviceAccountName"] = self.service_account_name



# API reference: https://metallb.universe.tf/apis/#metallb.io/v1beta1.IPAddressPool

from ocp_resources.resource import NamespacedResource


class IPAddressPool(NamespacedResource):
    """
    IPAddressPool object.
    """

    api_group = NamespacedResource.ApiGroup.METALLB_IO

    def __init__(
        self,
        name=None,
        namespace=None,
        client=None,
        addresses=None,
        auto_assign=True,
        avoid_buggy_ips=False,
        **kwargs,
    ):
        """
        Args:
            name (str): Name of the MetalLB or it's CR's.
            namespace (str): Namespace of the MetalLB
            client: (DynamicClient): DynamicClient to use.
            addresses (list): List of IP addresses.
            auto_assign (bool): IP address assignment. (default: True)
            avoid_buggy_ips (bool): Avoid buggy IP address (default: False)
        """
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            **kwargs,
        )
        self.addresses = addresses
        self.auto_assign = auto_assign
        self.avoid_buggy_ips = avoid_buggy_ips

    def to_dict(self):
        super().to_dict()
        if not self.yaml_file:
            if not self.addresses:
                raise ValueError(
                    "Either required parameter is missing 'addresses' or provide yaml_file."
                )

            self.res["spec"] = {"addresses": self.addresses}

            if self.auto_assign:
                self.res["spec"]["autoAssign"] = self.auto_assign

            if self.avoid_buggy_ips:
                self.res["spec"]["avoidBuggyIPs"] = self.avoid_buggy_ips



# https://kubernetes.io/docs/reference/kubernetes-api/workload-resources/controller-revision-v1/
from ocp_resources.constants import TIMEOUT_4MINUTES
from ocp_resources.resource import NamespacedResource


class ControllerRevision(NamespacedResource):
    api_group = NamespacedResource.ApiGroup.APPS

    def __init__(
        self,
        name=None,
        namespace=None,
        client=None,
        teardown=True,
        timeout=TIMEOUT_4MINUTES,
        privileged_client=None,
        yaml_file=None,
        delete_timeout=TIMEOUT_4MINUTES,
        owner_references=None,
        revision_object=None,
        **kwargs,
    ):
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            timeout=timeout,
            privileged_client=privileged_client,
            yaml_file=yaml_file,
            delete_timeout=delete_timeout,
            **kwargs,
        )
        self.owner_references = owner_references
        self.revision_object = revision_object

    def to_dict(self):
        super().to_dict()
        if not self.yaml_file:
            if self.owner_references:
                self.res.setdefault("metadata", {}).update(
                    {"ownerReference": self.owner_references}
                )
            if self.revision_object:
                self.res.update({"data": self.revision_object.res})



# -*- coding: utf-8 -*-

from ocp_resources.resource import NamespacedResource


class Backup(NamespacedResource):
    """
    Backup object.
    """

    api_group = NamespacedResource.ApiGroup.VELERO_IO

    def __init__(
        self,
        name=None,
        namespace=None,
        included_namespaces=None,
        client=None,
        teardown=False,
        privileged_client=None,
        yaml_file=None,
        excluded_resources=None,
        **kwargs,
    ):
        if not included_namespaces:
            raise ValueError("included_namespaces can't be None")

        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            privileged_client=privileged_client,
            yaml_file=yaml_file,
            **kwargs,
        )
        self.included_namespaces = included_namespaces
        self.excluded_resources = excluded_resources

    def to_dict(self):
        super().to_dict()
        if not self.yaml_file:
            self.res.update(
                {
                    "spec": {
                        "includedNamespaces": self.included_namespaces,
                    }
                }
            )

            if self.excluded_resources:
                self.res["spec"]["excludedResources"] = self.excluded_resources



from kubernetes.dynamic.exceptions import ForbiddenError
from openshift.dynamic.exceptions import (
    InternalServerError,
    NotFoundError,
    ServerTimeoutError,
)
from urllib3.exceptions import MaxRetryError, ProtocolError

DEFAULT_CLUSTER_RETRY_EXCEPTIONS = {
    MaxRetryError: [],
    ConnectionAbortedError: [],
    ConnectionResetError: [],
    InternalServerError: [
        "etcdserver: leader changed",
        "etcdserver: request timed out",
        "Internal error occurred: failed calling webhook",
        "rpc error:",
    ],
    ServerTimeoutError: [],
    ForbiddenError: ["context deadline exceeded"],
}
PROTOCOL_ERROR_EXCEPTION_DICT = {ProtocolError: []}
NOT_FOUND_ERROR_EXCEPTION_DICT = {NotFoundError: []}
TIMEOUT_1MINUTE = 60
TIMEOUT_4MINUTES = 240
